{--------------------------------------------------------------------

 Author:      J.M.Wehlou

 Description:
  This unit is supposed to be doing something, for instance file I/O,
  that needs to be measured. We're just faking it here, but the point
  of having this unit is to show how we connect to it.
  The "data" is provided by the main form, just to get the example running.

  There's one thing you have to watch out for in particular. If you register
  a listener using setClientMonitoredPerf(), you have to call it again with
  a nil argument before freeing the class that implements the
  IClientMonitoredPerf interface, else you'll get an access violation when
  the gCMP variable goes out of scope and tries to call _Release on your
  class. Also note that your class that implements IClientMonitoredPerf
  should not be reference counted (see PerfMonClient.pas).

 --------------------------------------------------------------------}

unit ClientMonitoredUnit;

interface

type
  // if you want this unit to call you with updated performance
  // counts, you implement this interface and call this unit
  // with your interface pointer (setClientMonitoredPerf())
  IClientMonitoredPerf = interface
    procedure CMRawCount(uVal: cardinal);
    procedure CMDeltaCount(uVal: cardinal);
  end;

  TClientMonitored = class
    procedure IncomingRawCount(uRC: cardinal);
    procedure IncomingDeltaCount(uDC: cardinal);
  end;

  procedure setClientMonitoredPerf(cmp: IClientMonitoredPerf);

// ==================================================================

implementation

var
  // there's only one possible listener for performance calls
  gCMP    : IClientMonitoredPerf;

// ------------------------------------------------------------------

procedure setClientMonitoredPerf(cmp: IClientMonitoredPerf);
begin
  gCMP := cmp;
end;

// ==================================================================
{ TClientMonitored }

procedure TClientMonitored.IncomingDeltaCount(uDC: cardinal);
begin
  // here, we're just outputting the value we get in, but try to use
  // your imagination: we get a string we write to a file and we output
  // the count of all strings we have output so far.

  // only output performance data if anyone is listening to them.
  if assigned(gCMP) then
    gCMP.CMDeltaCount(uDC);

  // and here you do whatever this unit is supposed to do in life...
end;

// ------------------------------------------------------------------

procedure TClientMonitored.IncomingRawCount(uRC: cardinal);
begin
  // see above comments and use your imagination for the value you output.
  if assigned(gCMP) then
    gCMP.CMRawCount(uRC);
end;

// ------------------------------------------------------------------

end.
