unit PerfMonClientMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls,
  ClientMonitoredUnit,
  PerfMonClientImpl, AppEvnts;


type
  TfrmPerfMonClient = class(TForm)
    btnInstall: TButton;
    btnUninstall: TButton;
    lblRawCounter: TLabel;
    lblDeltaCounter: TLabel;
    stRawCounter: TStaticText;
    stDeltaCounter: TStaticText;
    tmr: TTimer;
    ae: TApplicationEvents;
    procedure tmrTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure aeIdle(Sender: TObject; var Done: Boolean);
    procedure btnInstallClick(Sender: TObject);
    procedure btnUninstallClick(Sender: TObject);
  private
    { Private declarations }
    fuRawCounter    : cardinal;
    fuDeltaCounter  : cardinal;
    fCM             : TClientMonitored;
    fPMCI           : IPerfMonClientImpl;
  public
    { Public declarations }
  end;

var
  frmPerfMonClient: TfrmPerfMonClient;

// ==================================================================

implementation

{$R *.dfm}

uses
  Math;

// ------------------------------------------------------------------

procedure TfrmPerfMonClient.tmrTimer(Sender: TObject);
begin
  // the raw counter is normally used for absolute counts, such as
  // number of I/O requests queued to disk, so it goes up and down,
  // usually returning to zero
  fuRawCounter := Floor(sin(Time * 14400) * 50) + 50;
  stRawCounter.Caption := IntToStr(fuRawCounter);
  fCM.IncomingRawCount(fuRawCounter);

  // the delta counter is monotonally increasing and used for things
  // like total I/O requests. the performance counter application in
  // windows will show the number changed per time unit, usually per
  // second
  fuDeltaCounter := Floor(Random(100));
  stDeltaCounter.Caption := IntToStr(fuDeltaCounter);
  fCM.IncomingDeltaCount(fuDeltaCounter);

end;

// ------------------------------------------------------------------

procedure TfrmPerfMonClient.FormCreate(Sender: TObject);
begin
  fCM := TClientMonitored.Create();
  fPMCI := getPerfMonClientImpl();
end;

// ------------------------------------------------------------------

procedure TfrmPerfMonClient.FormDestroy(Sender: TObject);
begin
  fCM.Free;
end;

// ------------------------------------------------------------------

procedure TfrmPerfMonClient.aeIdle(Sender: TObject; var Done: Boolean);
begin
  btnUninstall.Enabled := fPMCI.IsInstalled;
  btnInstall.Enabled := not btnUninstall.Enabled; 
end;

// ------------------------------------------------------------------

procedure TfrmPerfMonClient.btnInstallClick(Sender: TObject);
begin
  fPMCI.Install();
end;

// ------------------------------------------------------------------

procedure TfrmPerfMonClient.btnUninstallClick(Sender: TObject);
begin
  fPMCI.Uninstall();
end;

// ------------------------------------------------------------------

end.
