{--------------------------------------------------------------------

 Author:      J.M.Wehlou

 Description: This DLL gets called by the operating system whenever
              a user starts Windows Performance Monitor and selects
              counters from our application.


 --------------------------------------------------------------------}


library PerfMonDLL;


uses
  SysUtils,
  WinTypes,
  Classes,
  Registry,
  PerfMonClientDef,
  PerfMonClientDefIf,
  PerfCounters,
  WinPerf,
  MemMappedFile;

{$R *.res}

var
  gMMF            : TMemMappedFile;
  guFirstCounter  : cardinal;
  guLastCounter   : cardinal;
  guFirstHelp     : cardinal;
  guLastHelp      : cardinal;
  gCounters       : TPerfCounters;

// ==================================================================

function PerfData_Open(pws: LPWSTR): DWORD; stdcall;
var
  sKey    : string;
  reg     : TRegistry;
  i       : integer;
  pmcd    : IPerfMonClientDef;
begin
  try
    reg := TRegistry.Create();
    try
      reg.RootKey := HKEY_LOCAL_MACHINE;
      sKey := getPerfMonClientDef().ServiceKey() + '\Performance';
      if reg.OpenKey(sKey, False) then begin
        guFirstCounter  := reg.ReadInteger('First Counter');
        guFirstHelp     := reg.ReadInteger('First Help');
        guLastCounter   := reg.ReadInteger('Last Counter');
        guLastHelp      := reg.ReadInteger('Last Help');
      end;
    finally
      reg.Free;
    end;

    gCounters := TPerfCounters.Create(guFirstCounter, guFirstHelp, PERF_DETAIL_NOVICE, 0);
    pmcd := getPerfMonClientDef();
    for i := 0 to pmcd.NbOfObjects() - 1 do begin
      case pmcd.ObjectCounterType(i) of
      PERF_COUNTER_RAWCOUNT :
        gCounters.AddRawCounter(guFirstCounter + 2 * (i+1), guFirstHelp + 2 * (i+1), pmcd.ObjectScale(i), PERF_DETAIL_NOVICE, 0);
      PERF_COUNTER_COUNTER :
        gCounters.AddDeltaCounter(guFirstCounter + 2 * (i+1), guFirstHelp + 2 * (i+1), pmcd.ObjectScale(i), PERF_DETAIL_NOVICE, 0);
      end;
    end;

    gMMF := TMemMappedFile.Create(pmcd.MemMappedFileName(), pmcd.TotalSize());

    Result := ERROR_SUCCESS;
  except
    on e: Exception do begin
      // here you should log the error somehow
    end;
  end;
end;

// ------------------------------------------------------------------

function PerfData_Close(): DWORD; stdcall;
begin
  try
    gMMF.Free;
    gCounters.Free;
    Result := ERROR_SUCCESS;
  except
    on e: Exception do begin
      // here you should log the error somehow
    end;
  end;
end;

// ------------------------------------------------------------------

function PerfData_Collect(pwsValueName: LPWSTR; var pvData: pointer;
          var cbTotalBytes: DWORD; var dwNumObjectTypes: DWORD): DWORD; stdcall;
var
  i     : integer;
  dw    : DWORD;
begin
  { in this implementation I don't give a damn about the value name,
    since it doesn't seem to be correct anyway. I think the best
    strategy is to always return all counters you have. See the
    article text for details. }

  try
    for i := 0 to gCounters.GetCount() - 1 do begin
      dw := gMMF.ReadDWORD(i * sizeof(cardinal));
      gCounters.SetValue(i, dw);
    end;

    gCounters.Collect(pvData, cbTotalBytes);
    dwNumObjectTypes := 1;

    Result := ERROR_SUCCESS;
  except
    on e: Exception do begin
      // here you should log the error somehow
    end;
  end;
end;

// ------------------------------------------------------------------

exports
  PerfData_Open,
  PerfData_Close,
  PerfData_Collect;

// ------------------------------------------------------------------

begin
end.
